#include <stdio.h>
#include <string.h> /* strlen() */
#include <time.h>
#include "calendar.h"
#include "xconio.h"

#define HIGHLIGHT(str) { printf("\33[47;5;30m%s\33[0m", str); }

static short print_day(short year, short month, short day, short cntr);

void Display(const short year,
             const short mon,
             const short days,
             const unsigned short nyd,
             short *gapPtr)
{
    const char *dayNames[]
        = { "Sun.", "Mon.", "Tues.", "Wed.", "Thur.", "Fri.", "Sat." };

    const char *months[] = { "January",   "February", "March",    "April",
                             "May",       "June",     "July",     "August",
                             "September", "October",  "November", "December" };

    short cntr = 1, day_cntr = 0, gap = *gapPtr, i, j;

    /* clean up console window -- TODO use the curses library */
    clrscr();

    /* title calendar with month and year */
    printf("\n\n%27s %hi\n\n", *(months + (mon - 1)), year);

    /* title columns with weekdays */
    for (i = 0; i < 7; i++)
        printf("%5s\t", *(dayNames + i));

    putchar('\n');

    /* yes, 6, in case a 31-day month starts on Sat. or Sun.*/
    for (i = 0; i < 6; i++) {
        /* skip the weekdays before the start of the month */
        while (day_cntr == 0 && gap > 0) {
            printf("%3c\t", 0);
            gap--;
        }

        /* restore gap after decrementing to 0 */
        gap = GetGap(year, mon, nyd);
        *gapPtr = gap;

        /* start of the first week?; then start printing days up to the end of
         * first week */
        if (day_cntr == 0) {
            for (j = cntr; j < (7 - gap) + cntr; j++)
                day_cntr = print_day(year, mon, j, day_cntr);
        }
        /* the start of week 2 or later?; then print a full week */
        else {
            for (j = cntr; j < 7 + cntr; j++) {
                day_cntr = print_day(year, mon, j, day_cntr);

                if (j >= days) /**< last day of month reached */
                    break;
            }
        }

        /* first week completed b/c we skipped some weekdays jump to start of
         * next week, remembering how many weekdays we skipped */
        if (day_cntr < 7)
            cntr += 7 - gap;
        /* completed a full week, or the first week began on Sun., so no days
         * were skipped */
        else if (day_cntr > 6 && day_cntr < days)
            cntr += 7; /* jump to start of next week */
        else
            break;

        putchar('\n');
    }

    /* show navigation options */
    if (mon == 12) {
        if (year < 2099)
            printf("\n\n Go to %9s %hi . . . %-7s",
                   *(months),
                   year + 1,
                   "> arrow"); /**< go to Jan. of next yr */
        else                   /**< end of year range */
            putchar('\n');

        printf("\n Go to %9s %hi . . . %-7s",
               *(months + 10),
               year,
               "< arrow"); /* go to Nov. of current yr */
        if (year < 2099)
            printf("\n Go to %9s %hi . . . %-7s",
                   *(months + 11),
                   year + 1,
                   "^ arrow"); /* same month, next yr*/
        if (year > 2001)
            printf("\n Go to %9s %hi . . . %-7s",
                   *(months + 11),
                   year - 1,
                   "v arrow"); /* same month, prev yr*/
    }
    else {
        printf("\n\n Go to %9s %hi . . . %-7s",
               *(months + mon),
               year,
               "> arrow"); /* next month*/
        if (mon == 1) {
            if (year > 2001)
                printf("\n Go to %9s %hi . . . %-7s",
                       *(months + 11),
                       year - 1,
                       "< arrow"); /* from Jan to Dec*/
        }
        else
            printf("\n Go to %9s %hi . . . %-7s",
                   *(months + (mon - 2)),
                   year,
                   "< arrow"); /* prev month */

        if (year < 2099)
            printf("\n Go to %9s %hi . . . %-7s",
                   *(months + (mon - 1)),
                   year + 1,
                   "^ arrow"); /* same month, next yr */
        if (year > 2001)
            printf("\n Go to %9s %hi . . . %-7s",
                   *(months + (mon - 1)),
                   year - 1,
                   "v arrow"); /* same month, prev yr */
    }

    printf("\n %-18s . . . . . . . %-7s", "Set month and year", "R");
    printf("\n %-3s . . . . . . . . . . . . . . %-7s\n", "Exit", "Q");
}

static short print_day(short year, short month, short day, short cntr)
{
    time_t now;
    struct tm *t;
    char buf[6] = { 0 };

    time(&now);
    t = localtime(&now);
    sprintf(buf, "%3hi", day);

    if (strlen(buf) == 5)
        buf[5] = '\0';
    else
        buf[4] = '\0';

    if (t->tm_mday == day
            && (t->tm_mon + 1) == month
            && (1900 + t->tm_year) == year) {
        HIGHLIGHT(buf)
        putchar('\t');
    }
    else
        printf("%3hi\t", day);

    return ++cntr;
}
