#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include "calendar.h"
#include "xconio.h"

static void set_date(short *yrPtr,
                     short *monPtr,
                     unsigned char *needMonthPtr,
                     unsigned char *needYearPtr);

unsigned char Menu(short *yrPtr, short *monPtr, unsigned char *retry)
{
    unsigned char needMonth = FALSE, needYear = FALSE, stop = FALSE;
    short mon = *monPtr;
    short year = *yrPtr;
    short choice = getkey();

    switch (choice) {
#if defined(_WIN32) || defined(__WIN32__)
    case 77: /* Win/DOS value of 2nd char sent to buffer by RIGHT arrow key */
#else
    case 67: /* POSIX value of 3rd char sent to buffer by RIGHT arrow key */
#endif
        if (mon == 12) {
            if (year + 1 > 2099) {
                printf("\n%37s", "Year can't exceed 2099!");
                *retry = TRUE;
                break;
            }
            else {
                *monPtr = 1;
                ++*yrPtr; /* increment the value BEFORE it's read */
            }
        }
        else
            ++*monPtr;

        *retry = FALSE;
        break;
#if defined(_WIN32) || defined(__WIN32__)
    case 75: /* LEFT */
#else
    case 68:
#endif
        if (mon == 1) {
            if (year - 1 < 2001) {
                printf("\n%42s", "Year can't be earlier than 2001!");
                *retry = TRUE;
                break;
            }
            else {
                *monPtr = 12;
                --*yrPtr;
            }
        }
        else
            --*monPtr;

        *retry = FALSE;
        break;
#if defined(_WIN32) || defined(__WIN32__)
    case 72: /* UP */
#else
    case 65:
#endif
        if (year + 1 > 2099) {
            printf("\n%37s", "Year can't exceed 2099!");
            *retry = TRUE;
            break;
        }
        else
            ++*yrPtr;

        *retry = FALSE;
        break;
#if defined(_WIN32) || defined(__WIN32__)
    case 80: /* DOWN */
#else
    case 66:
#endif
        if (year - 1 < 2001) {
            printf("\n%42s", "Year can't be earlier than 2001!");
            *retry = TRUE;
            break;
        }
        else
            --*yrPtr;

        *retry = FALSE;
        break;
    case 114:
    case 82:
        clrscr();
        needMonth = TRUE;
        needYear = TRUE;
        set_date(yrPtr, monPtr, &needMonth, &needYear);
        *retry = FALSE;
        break;
    case 113:
    case 81:
        stop = TRUE;
        clrscr();
        break;
    default:
        if (needMonth || needYear) {
            clrscr();
            set_date(yrPtr, monPtr, &needMonth, &needYear);
            *retry = FALSE;
        }
        else
            *retry = TRUE;

        gotoxy(2, 20);
        printf("Try another key . . . ");
        break;
    }

    return (stop);
}

static inline void chomp(char(*pStr))
{
    size_t i, len = strlen(pStr) + 1;

    if (pStr) {
        for (i = 0; i < len; i++) {
            if (i < len - 1 && (*(pStr + i) == '\r' || *(pStr + i) == '\n')) {
                *(pStr + i) = '\0';
                break;
            }
        }
    }
}

static void set_date(short *yrPtr,
                     short *monPtr,
                     unsigned char *needMonthPtr,
                     unsigned char *needYearPtr)
{
    short year = *yrPtr, mon = *monPtr;
    unsigned char needMonth = *needMonthPtr, needYear = *needYearPtr;
    char *nl;

    while (needMonth || needYear) {
        char buf[BUFSIZ] = { 0 };
        short out = 0;

        if (needMonth) {
            printf("%30s", "\nChoose a month (1-12):   ");
            fgets(buf, sizeof buf, stdin);
            chomp(buf);
            out = (short)strtoul(buf, &nl, 10);

            if (errno != 0 || strlen(buf) > 2 || out < 1 || out > 12) {
                puts("\nMonth must be between 1 (Jan.) and 12 (Dec.)");
                clearerr(stdin);
                errno = 0;
                continue;
            }

            else {
                *monPtr = mon = out;
                *needMonthPtr = needMonth = FALSE;
            }

            memset(buf, 0, sizeof buf);
            out = 0;
        }

        if (needYear) {
            printf("%25s", "\nChoose a year (yyyy):    ");
            fgets(buf, sizeof buf, stdin);
            chomp(buf);
            out = (short)strtoul(buf, &nl, 10);

            if (errno != 0 || strlen(buf) > 4 || out < 2001 || out > 2099) {
                /** accuracy limitation of chosen date calc formula.
                 * @see DateCalc */
                puts("\nYear must be within following range: 2001-2099.");
                clearerr(stdin);
                errno = 0;
                continue;
            }
            else {
                *yrPtr = year = out;
                *needYearPtr = needYear = FALSE;
            }
        }
    }
}
